# PiconCleaner
#
# Coded by dre (c) 2014
# Support: www.dreambox-tools.info
# E-Mail: dre@dreambox-tools.info
#
# This plugin is open source but it is NOT free software.
#
# This plugin may only be distributed to and executed on hardware which
# is licensed by Dream Multimedia GmbH.
# In other words:
# It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
# to hardware which is NOT licensed by Dream Multimedia GmbH.
# It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
# on hardware which is NOT licensed by Dream Multimedia GmbH.
#
# If you want to use or modify the code or parts of it,
# you have to keep MY license and inform me about the modifications by mail.

from Components.ActionMap import ActionMap, HelpableActionMap
from Components.ConfigList import ConfigList, ConfigListScreen
from Components.config import config, ConfigSubsection, ConfigText, configfile, ConfigSelection, getConfigListEntry, ConfigYesNo
from Components.Label import Label
from Components.MenuList import MenuList
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaTest
from Components.Pixmap import Pixmap
from Screens.MessageBox import MessageBox
from Plugins.Plugin import PluginDescriptor
from Screens.ChannelSelection import ChannelSelection
from Screens.Screen import Screen
from Screens.HelpMenu import HelpableScreen
from Tools.Directories import fileExists, SCOPE_SKIN_IMAGE, SCOPE_CURRENT_SKIN, resolveFilename, pathExists

from os import listdir as os_listdir, system as os_system
from enigma import eServiceCenter, eServiceReference, eListboxPythonMultiContent, gFont, loadPNG, eEnv, RT_HALIGN_RIGHT

class PiconMenuList(MenuList):
	def __init__(self, list, selection = 0):
		MenuList.__init__(self, list, False, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 22))
		self.l.setFont(1, gFont("Regular", 18))
		self.l.setItemHeight(100)
		self.selection = selection
		
	def postWidgetCreate(self, instance):
		MenuList.postWidgetCreate(self, instance)
		self.moveToIndex(self.selection)
		
	def setSelectedIndex(self, index):
		self.moveToIndex(index)
		
def PiconViewerListEntry(path, sref):
	res = [ (path, sref) ]

	if sref is None:
		picon = resolveFilename(SCOPE_CURRENT_SKIN, "picon_default.png")
	else:
		picon = path + sref + ".png"

	res.append(MultiContentEntryPixmapAlphaTest(pos=(0,0), size=(240,100), png=loadPNG(picon)))
	#res.append(MultiContentEntryText(pos=(110,40), size=(140,20), font=1, text=_("Service Reference:")))
	
	res.append(MultiContentEntryText(pos=(250,40), size=(320,20), font=1, text=sref, flags=RT_HALIGN_RIGHT)) 	
	
	return res

def main(session,**kwargs):
	session.open(PiconCleaner)

def Plugins(**kwargs):

	list = []
	list.append(PluginDescriptor(name="PiconCleaner", description=_("Clean up unused picons"), where = [PluginDescriptor.WHERE_PLUGINMENU], fnc=main))
	return list

class PiconCleaner(Screen, HelpableScreen):
	skin = """
		<screen position="center,center" size="600,475" title="PiconCleaner">
			<widget name="menulist" position="0,0" size="600,400" scrollbarMode="showOnDemand" />
			<widget name="ButtonRedtext" position="50,445" size="300,20" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonRed" pixmap="skin_default/buttons/button_red.png" position="10,445" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonBluetext" position="50,415" size="250,20" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonBlue" pixmap="skin_default/buttons/button_blue.png" position="10,415" zPosition="10" size="16,16" transparent="1" alphatest="on" />
		</screen>
		"""

	def __init__(self, session, args=0):
		self.skin = PiconCleaner.skin
		self.session = session
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"cancel":	(self.close, _("Close")),
		}, -1)

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":	(self.deleteSelectedPicon, _("Delete selected picon")),
			"blue": (self.deleteAllPicons, _("Delete all redundant picons")),
		}, -1)		
		
		self["ButtonBlue"] = Pixmap()
		self["ButtonBluetext"] = Label(_("Delete all redundant picons"))
		
		self["ButtonRed"] = Pixmap()
		self["ButtonRedtext"] = Label(_("Delete selected picon"))
		
		self["menulist"] = PiconMenuList([])
		
		self.PiconEntryList = []
		self.allServicesList = []
		self.markerBouquetList = []
		self.stringServicesList = []
		
		self.onShown.append(self.buildScreen)
	
	def buildScreen(self):
		serviceslist = self.getBouquets()
		
		self.PiconMappingList = []
		
		for path in (eEnv.resolve('${datadir}/enigma2/picon/'),'/media/cf/picon/','/media/usb/picon/','/data/picon/',eEnv.resolve('${datadir}/enigma2/picon_50x30/'),'/media/cf/picon_50x30/','/media/usb/picon_50x30/','/data/picon_50x30/', eEnv.resolve('${datadir}/enigma2/picon_220x132/'),'/media/cf/picon_220x132/','/media/usb/picon_220x134/','/data/picon_220x134/', eEnv.resolve('${datadir}/enigma2/picon_320x240/'),'/media/cf/picon_320x240/','/media/usb/picon_320x240/','/data/picon_320x240/', eEnv.resolve('${datadir}/enigma2/picon_500x300/'),'/media/cf/picon_500x300/','/media/usb/picon_500x300/','/data/picon_500x300/'):
			if pathExists(path):
				for file in os_listdir(path):
					pos = file.find(".png")
					if pos != -1:
						filename = file[:pos]
						
						if not filename in serviceslist and not filename in self.markerBouquetList:
							self.PiconMappingList.append(PiconViewerListEntry(path, filename))
			
		self["menulist"].setList(self.PiconMappingList)
		
	def deleteSelectedPicon(self):
		self.index = self["menulist"].getSelectedIndex()
		
		file = "%s%s.png" %(self.PiconMappingList[self.index][0][0], self.PiconMappingList[self.index][0][1])
		
		file = file.replace("(", "\(").replace(")", "\)")
		
		self.error = os_system("rm -f %s" %(file))
		if (int(self.error)/256)==0:
			self.session.open(MessageBox, _("%s%s.png was deleted") %(self.PiconMappingList[self.index][0][0], self.PiconMappingList[self.index][0][1]), MessageBox.TYPE_INFO, timeout=5)
		else:
			self.session.open(MessageBox, _("File was not deleted due to an error") %(self.error), MessageBox.TYPE_ERROR, timeout=5)
		
		self.buildScreen()

	def deleteAllPicons(self):
		message = ""
		errors = 0
		oks = 0
		for entry in self.PiconMappingList:
			file = "%s%s.png" %(entry[0][0], entry[0][1])

 			message += "%s\n" %(file)

			file = file.replace("(", "\(").replace(")", "\)")	
			self.error = os_system("rm -f %s" %(file))
			
			if (int(self.error)/256)==0:
				oks += 1
			else:
				errors += 1
				
		self.session.open(MessageBox, _("%d of files were deleted\n%d files were not deleted due to an error") %(oks, errors), MessageBox.TYPE_INFO, timeout=5)

	def getBouquetServices(self, bouquetRoot):
		bouquets = []
		
		self.eServiceCenterInstance = eServiceCenter.getInstance()
		if config.usage.multibouquet.value:
			serviceList = self.eServiceCenterInstance.list(bouquetRoot)
			if serviceList:
				while True:
					service = serviceList.getNext()
					if not service.valid():
						break
					if service.flags & eServiceReference.isDirectory:
						info = self.eServiceCenterInstance.info(service)
						if info:
							bouquets.append(service)
							self.markerBouquetList.append(info.getName(service).replace(" ", "_"))
				return bouquets
		else:
			info = self.eServiceCenterInstance().info(bouquetRoot)
			if info:
				bouquets.append(bouquetRoot)
			return bouquets
	
	def getBouquets(self):		
		from Screens.ChannelSelection import service_types_tv, service_types_radio
		bouquetStringList = []
		if config.usage.multibouquet.value:
			bouquetStringList.append(service_types_tv + ' FROM BOUQUET "bouquets.tv"')
			bouquetStringList.append(service_types_radio + ' FROM BOUQUET "bouquets.radio"')
		else:
			bouquetStringList.append(service_types_tv + ' FROM BOUQUET "userbouquet.favourites.tv"')
			bouquetStringList.append(service_types_radio + ' FROM BOUQUET "userbouquet.favourites.radio"')
			
		for bouquetString in bouquetStringList:
			bouquetRoot = eServiceReference(bouquetString)
			bouquetServices = self.getBouquetServices(bouquetRoot)
			if bouquetServices is None:
				continue
				
			for service in bouquetServices:
				self.getDirectoryServices(service)
			
			for service in self.allServicesList:
				sRef = self.buildServiceRefString(service)

				if sRef is "":
					continue

				self.stringServicesList.append(sRef)	

		return self.stringServicesList				
				
					
		
	def getDirectoryServices(self, directory):
		serviceList = self.eServiceCenterInstance.list(directory)
		
		if serviceList:
			while True:
				service = serviceList.getNext()
				oldPath = ""
				if service.getPath():
					oldPath = service.getPath()
					service.setPath("")
				if not service.valid():
					service.setPath(oldPath)
					break
				if service.flags & eServiceReference.isGroup:
					self.getDirectoryServices(service)
				elif service.flags & eServiceReference.isMarker:
					self.markerBouquetList.append(service.getName().replace(" ", "_"))
				else:
					self.allServicesList.append(service)
				service.setPath(oldPath)

	def buildServiceRefString(self,service):
		sRef = service.toString()
		if sRef is "":
			return sRef

		# strip all after last http (e.g. partnerbox-services)
		pos = sRef.rfind("http")
		if pos != -1:
			sRef = sRef[:pos]

		pos = sRef.rfind(':')
		if pos == -1:
			return ""

		sRef = sRef[:pos].rstrip(':').replace(':','_')
		return sRef